package org.processmining.generalizedconformance.algorithms.heuristic.footprint;


import org.jbpt.bp.RelSet;
import org.jbpt.bp.RelSetType;
import org.jbpt.bp.construct.AbstractRelSetCreator;
import org.processmining.generalizedconformance.algorithms.heuristic.processtree.NodeWrapper;
import org.processmining.generalizedconformance.algorithms.heuristic.processtree.TreeWrapper;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task.Manual;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;



public class ProcessTreeFootprintCreator extends AbstractRelSetCreator {
	
	private static ProcessTreeFootprintCreator eInstance;
	
	public static ProcessTreeFootprintCreator getInstance() {
		if (eInstance == null)
			eInstance  = new ProcessTreeFootprintCreator();
		return eInstance;
	}
	
	private ProcessTreeFootprintCreator() {
		
	}
	
	public RelSet<ProcessTree, NodeWrapper> deriveFootprint(ProcessTree pt) {
		ProcessTree wrappedTree = TreeWrapper.getWrappedTree(pt);
		Set<NodeWrapper> leafNodes = new HashSet<NodeWrapper>();
		for (Node n : wrappedTree.getNodes())
			if (n instanceof NodeWrapper){
				NodeWrapper wrappedNode = (NodeWrapper) n;
				if (wrappedNode.isLeaf() && wrappedNode.getRealNode() instanceof Manual)
					leafNodes.add(wrappedNode);	
			}
			
		return deriveFootprint(wrappedTree, new ArrayList<NodeWrapper>(leafNodes));
	}
	
	public RelSet<ProcessTree, NodeWrapper> deriveFootprint(ProcessTree pt, Collection<NodeWrapper> nodes) {

		ProcessTreeFootprintHandler processTreeHandler = new ProcessTreeFootprintHandler(pt);
		
		RelSet<ProcessTree, NodeWrapper> profile = new RelSet<ProcessTree, NodeWrapper>(pt,nodes);
		RelSetType[][] matrix = profile.getMatrix();

		for(NodeWrapper t1 : profile.getEntities()) {
			int index1 = profile.getEntities().indexOf(t1);
			for(NodeWrapper t2 : profile.getEntities()) {
				int index2 = profile.getEntities().indexOf(t2);
				
				/*
				 * The matrix is symmetric. Therefore, we 
				 * need to traverse only half of the entries.
				 */
				if (index2 > index1)
					continue;

				//System.out.println("Checking: " + t1 + " " + t2);
				
				boolean t1CanDirectlyFollowT2 = processTreeHandler.canDirectlyFollow(t1, t2);
				boolean t2CanDirectlyFollowT1 = processTreeHandler.canDirectlyFollow(t2, t1);
				
				
				if (t1CanDirectlyFollowT2 && t2CanDirectlyFollowT1) {
					super.setMatrixEntry(matrix, index1, index2, RelSetType.Interleaving);
				}
				else if (t1CanDirectlyFollowT2 || t2CanDirectlyFollowT1) {
					if (t1CanDirectlyFollowT2)
						super.setMatrixEntryOrder(matrix, index1, index2);
					else 
						super.setMatrixEntryOrder(matrix, index2, index1);
				}
				else {
					super.setMatrixEntry(matrix, index1, index2, RelSetType.Exclusive);
				}
			}
		}		
		
		
		return profile;
	}

}