package org.processmining.generalizedconformance.algorithms.heuristic.relset;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import com.google.gwt.dev.util.collect.HashSet;
import org.jbpt.bp.RelSet;
import org.jbpt.bp.RelSetType;
import org.processmining.generalizedconformance.algorithms.heuristic.processtree.NodeWrapper;
import org.processmining.processtree.ProcessTree;

import java.util.Set;

public class NameBasedSimilarity {
	/**
	 * Computes the name based simple similarity between two relation sets.
	 * 
	 * @param set1 {@link RelSet}
	 * @param set2 {@link RelSet}
	 * @return Jaccard coefficient of the matching relations for the intersection of their node sets, relative to the union of their node sets.
	 */
	public static double getNameBasedSimilarity(RelSet<ProcessTree, NodeWrapper> set1, RelSet<ProcessTree, NodeWrapper> set2){
		
		Set<String> names = new HashSet<>();
		
		BiMap<NodeWrapper, String> names1 = HashBiMap.create();
		BiMap<NodeWrapper, String> names2 = HashBiMap.create();
		for (NodeWrapper node : set1.getEntities()){
			names.add(node.getName());
			names1.put(node, node.getName());
		}
		for (NodeWrapper node : set2.getEntities()){
			names.add(node.getName());
			names2.put(node, node.getName());
		}
		int countMatches = 0;
		int countPotentialMatches = 0;
		for (String name1 : names){
			if (names1.inverse().containsKey(name1) && names2.inverse().containsKey(name1)){
				NodeWrapper n1 = names1.inverse().get(name1);
				NodeWrapper n2 = names2.inverse().get(name1);
				for (String name2 : names) {
					if (names1.inverse().containsKey(name2) && names2.inverse().containsKey(name2)){
						countPotentialMatches++;
						NodeWrapper o1 = names1.inverse().get(name2);
						NodeWrapper o2 = names2.inverse().get(name2);
						RelSetType firstType = set1.getRelationForEntities(n1, o1);
						RelSetType otherType = set2.getRelationForEntities(n2, o2);
						if (firstType.equals(otherType)){
							countMatches++;
						}
					}
				}
			}
		}
		// number of total matches divided by the union of their potential relation overlap
		return countMatches / (Math.pow(names1.size(),2) + Math.pow(names2.size(),2) - countPotentialMatches);
	}
}
