package org.processmining.generalizedconformance.algorithms.miner.alignment;

import java.util.ArrayList;
import java.util.List;

/**
 * For each node in the tree, this collects partial alignments through backwards replay of a tree record (alignment)
 * That is, whenever a node is enabled or children are enabled, their partial alignments get appended the current replayed piece.
 *
 * Created by andreas on 4/13/17.
 */
public class AlignmentCollector {
    private Alignment partialAlignment;

    private List<Alignment> collectedAlignments;

    public AlignmentCollector(){
        partialAlignment = null;
        collectedAlignments = new ArrayList<>();
    }

    public void collect(AlignmentPiece piece, int frequency){
        if (partialAlignment == null){
            partialAlignment = new Alignment(frequency);
        }
        partialAlignment.addPiece(piece);
    }

    public void doneCollecting(){
        if (partialAlignment!=null && partialAlignment.size() > 0){
            Alignment alignment = new Alignment(partialAlignment);
            Alignment other = null;
            for (Alignment a : collectedAlignments){
                if (alignment.equals(a)){
                    other = a;
                    break;
                }
            }
            if (other!=null){
                other.incFrequency(alignment.getFrequency());
            } else {
                collectedAlignments.add(alignment);
            }
            partialAlignment = null;
        }
    }

    public List<Alignment> getCollectedAlignments() {
        return collectedAlignments;
    }
}
