package org.processmining.generalizedconformance.algorithms.miner.alignment;

/**
 * Created by andreas on 4/13/17.
 */
public class AlignmentPiece {
    private int modelMove;
    private int logMove;
    private byte[] marking;

    public AlignmentPiece(int modelMove, int logMove, byte[] marking) {
        this.modelMove = modelMove;
        this.logMove = logMove;
        this.marking = marking;
    }

    /**
     * encodes a leaf in the model according to the alignment
     * @return int number >= 0, when we do a model move (or synch. move when also log move >= 0 )
     * -1, if the model is not moving (log move)
     */
    public int getModelMove() {
        return modelMove;
    }

    /**
     * encodes a log move according to the alignment
     * @return a number >= 0, when we do a log move (or synch. move when also model move >= 0 )
     * -1, if the log is not moving (model move)
     */
    public int getLogMove() {
        return logMove;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        AlignmentPiece that = (AlignmentPiece) o;

        if (modelMove != that.modelMove) return false;
        return logMove == that.logMove;
    }

    public byte[] getMarking() {
        return marking;
    }

    @Override
    public int hashCode() {
        long result = modelMove;
        result = 31 * result + logMove;
        return (int)result;
    }

    @Override
    public String toString() {
        return "(m: " + modelMove +", l: " + logMove + ')';
    }
}
