package org.processmining.generalizedconformance.algorithms.miner.alignment;

import com.sun.org.apache.xml.internal.dtm.ref.DTMDefaultBaseIterators;
import gnu.trove.map.TIntIntMap;
import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TIntIntHashMap;
import gnu.trove.set.TIntSet;
import nl.tue.astar.AStarException;
import nl.tue.astar.AStarObserver;
import nl.tue.astar.AStarThread;
import nl.tue.astar.Trace;
import org.processmining.plugins.boudewijn.treebasedreplay.astar.ModelPrefix;
import org.processmining.plugins.boudewijn.treebasedreplay.astar.TreeMarkingVisit;
import org.processmining.plugins.etm.model.narytree.NAryTree;
import org.processmining.plugins.etm.model.narytree.replayer.basisilp.NAryTreeBasisILPDelegate;
import org.processmining.plugins.etm.model.narytree.replayer.*;
import org.processmining.plugins.etm.model.narytree.replayer.basisilp.NAryTreeBasisILPTail;
import org.processmining.plugins.etm.model.narytree.replayer.basisilp.NAryTreeReplayerWithBasisILP;
import org.processmining.plugins.etm.model.narytree.replayer.basisilp2.NAryTreeBasis2ILPDelegate;
import org.processmining.plugins.etm.model.narytree.replayer.basisilp2.NAryTreeReplayerWithBasis2ILP;
import org.processmining.plugins.etm.model.narytree.replayer.fullilp.NAryTreeFullILPDelegate;
import org.processmining.plugins.etm.model.narytree.replayer.fullilp.NAryTreeFullILPTail;
import org.processmining.plugins.etm.model.narytree.replayer.fullilp.NAryTreeReplayerWithFullILP;
import org.processmining.plugins.etm.model.narytree.replayer.hybridilp.NAryTreeHybridILPTail;

import javax.print.attribute.standard.MediaSize;
import java.util.Map;

/**
 * Created by andreas on 4/13/17.
 */
public class NAryTreeAnnotatorReplayer extends NAryTreeReplayerWithBasis2ILP {

    protected AlignmentAnnotator<NAryTreeHead, NAryTreeHybridILPTail> postProcessor;
    private TIntIntMap maxRepetions;

    public NAryTreeAnnotatorReplayer(AStarAlgorithm aStarLogAbstraction, AStarThread.Canceller canceller, NAryTree tree, int configurationNumber, int[] node2cost, Map<TreeMarkingVisit<ModelPrefix>, TIntSet> marking2modelmove, TObjectIntMap<TreeMarkingVisit<ModelPrefix>> marking2visitCount, int[] syncMoveCount, int[] aSyncMoveCount, int[] moveCount, Map<Trace, TreeRecord> alignments, boolean useOrRows, int nrThreads, AStarObserver... observers) {
        super(aStarLogAbstraction, canceller, tree, configurationNumber, node2cost, marking2modelmove, marking2visitCount,
                syncMoveCount, aSyncMoveCount, moveCount, alignments, useOrRows, observers);
        this.maxRepetions = new TIntIntHashMap();
        this.postProcessor = new AlignmentAnnotator<>(delegate, tree, configurationNumber, marking2modelmove,
                marking2visitCount, syncMoveCount, aSyncMoveCount, moveCount, alignments, maxRepetions);

    }
    protected int calculateCostAndPostProcess(VerboseLevel verbose, Trace trace, TreeRecord r, int frequency) {

        postProcessor.process(this, verbose, trace, r, frequency);

        int cost = frequency * r.getCostSoFar();
        TreeRecord rec = r;
        while (rec.getPredecessor() != null) {
            cost -= frequency * ((rec.getModelMove() < tree.size() ? 1 : 0) + rec.getInternalMovesCost());
            rec = rec.getPredecessor();
        }
        cost -= frequency * rec.getInternalMovesCost();
        return cost;
    }

    protected NAryTreeHead createInitialHead(Trace trace) {
        return new NAryTreeHead(delegate, trace);
    }

    protected NAryTreeBasis2ILPDelegate constructDelegate(AStarAlgorithm algorithm, NAryTree tree,
                                                                         int configurationNumber, int[] node2cost, int threads, boolean useOrRows) {
        return new NAryTreeBasis2ILPDelegate(algorithm, tree, configurationNumber, node2cost, threads, useOrRows);
    }

    public int run(VerboseLevel verbose, int stopAt, int minModelCost, double timeLimit) throws AStarException {
        try {
            return super.run(verbose, stopAt, minModelCost, timeLimit);
        } finally {
            delegate.deleteLPs();
        }

    }
    public AlignmentCollector[] getPartialAlignmentsPerNode() {
        return this.postProcessor.getPartialAlignmentsPerNode();
    }
    public NAryTree getTree(){
        return this.tree;
    }

    public TIntIntMap getMaxRepetions() {
        return maxRepetions;
    }
    public AStarAlgorithm getAStarAlgorithm(){
        return this.aStarLogAbstraction;
    }
}
