package org.processmining.generalizedconformance.algorithms.miner.move;

import gnu.trove.iterator.TIntIterator;
import gnu.trove.map.TObjectIntMap;
import nl.tue.astar.Trace;
import nl.tue.astar.util.LinearTrace;
import org.processmining.generalizedconformance.algorithms.alignment.LogAbstraction;
import org.processmining.generalizedconformance.algorithms.miner.OptimalMiner;

/**
 * Created by andreas on 4/13/17.
 */
public class LogMove<M> extends Move<M>{

    private final Trace variantId;
    private final int position;
    private final int insertId;

    /**
     * @param variantId the id of the trace variant to be changed
     * @param position the position of the event to be inserted or deleted from the log
     * @param insertId the id of the event to be inserted (-1 if it should be deleted)
     * @param frequency the count of this affected variant in the entire log.
     * @param logSize the size of the entire log
     */
    public LogMove(Trace variantId, int position, int insertId, int frequency, int logSize){
        super(frequency, logSize);
        this.variantId = variantId;
        this.position = position;
        this.insertId = insertId;
    }

    @Override
    protected double computeExpectedGain(int...arguments) {
        return arguments[0]/(double)arguments[1]; // expected gain = frequency / logSize
    }

    @Override
    public SearchState<M> apply(SearchState<M> state, OptimalMiner<?> miner) {
        LogAbstraction log = state.getLogAbstraction();
        int[] newTrace = new int[insertId>=0?variantId.getSize()+1:variantId.getSize()-1];
        TIntIterator iter = variantId.iterator();
        int pos = -1;
        while (iter.hasNext()){
            int current = iter.next();
            pos += 1;
            boolean insert = true;
            if (pos == position){
                if (insertId >= 0) {
                    newTrace[pos] = insertId;
                    pos += 1;
                } else {
                    insert = false;
                }
            }
            if (insert)
                newTrace[pos] = current;
        }
        Trace newT = new LinearTrace(variantId.getLabel(), newTrace);

        TObjectIntMap<Trace> mapping = miner.getTraceMapping().remove(variantId);

        int i = log.getConvertedLog().remove(variantId);
        log.getConvertedLog().adjustOrPutValue(newT,i,i);
        if (miner.getTraceMapping().containsKey(newT)){
            // merging
            miner.getTraceMapping().get(newT).putAll(mapping);
        } else {
            // adding
            miner.getTraceMapping().put(newT, mapping);
        }

        return new SearchState<>(log, state.getModel());
    }
}
