package org.processmining.generalizedconformance.algorithms.miner.move;

import org.processmining.generalizedconformance.algorithms.miner.OptimalMiner;

/**
 * Created by andreas on 4/13/17.
 */
public abstract class Move<M> {

    protected double expectedGain;
    protected double realGain;

    public Move(double expectedGain){
        this.expectedGain = expectedGain;
    }

    public Move(int...arguments){
        this.expectedGain = computeExpectedGain(arguments);
    }

    /**
     * The expected gain when applying the move.
     * This depends on the number of particular traces being corrected by this move (and in case the model moves, the
     * total number of affected traces)
     * @return double the expected gain which can be used to sort moves in the neighborhood and compute probabilities.
     */
    public final double getExpectedGain(){
        return expectedGain;
    }

    /**
     * @param arguments as supplied in the constructor
     * @return
     */
    protected abstract double computeExpectedGain(int...arguments);

    /**
     * Applies the move to the given search state. Subclasses need to implement this
     * @param state {@link SearchState} composed of current log and model
     * @return SearchState the neighbor search state after applying the move.
     */
    public abstract SearchState<M> apply(SearchState<M> state, OptimalMiner<?> miner);

    public double getRealGain() {
        return realGain;
    }
    public void setRealGain(double realGain) {
        this.realGain = realGain;
    }
}
