package org.processmining.generalizedconformance.algorithms.miner.move;

import org.processmining.generalizedconformance.algorithms.mover.TreeUtils;
import org.processmining.plugins.etm.model.narytree.NAryTree;

/**
 * Created by andreas on 4/24/17.
 */
public class Operation {
    /**
     * removes a leaf from the tree
     * exp.gain: Model moves / synchronous moves
     */
    public static final short REMOVE = 0;
    /**
     * makes a node optional by adding an optional tau/node at that point
     * exp. gain: Log moves / synchronous moves
     */
    public static final short INSERT = 1;

    /**
     * Converts a connector to a loop/sequence/XOR/etc. node
     */
    public static final short CONVERT_TO = 2;

    protected short op;
    protected short node = NAryTree.TAU;
    protected short connector = NAryTree.SEQ;
    protected boolean left = false; // left or right

    public Operation(short op) {
        this (op, NAryTree.SEQ);
    }

    public Operation(short op, short connector) {
        this(op, connector, false);
    }

    public Operation(short op, short connector, boolean left) {
        this.op = op;
        this.connector = connector;
        this.left = left;
    }

    /**
     * Can store a node type. For example, which not to insert in parallel or as exclusive choice
     * @return
     */
    public short getNode(){
        return node;
    }

    public void setNode(short node) {
        this.node = node;
    }

    /**
     * @return position of insertion in the list of children
     */
    public boolean getLeft() {
        return left;
    }

    public short getOp(){
        return op;
    }

    public short getConnector() {
        return connector;
    }

    @Override
    public String toString() {
        String name;
        String nodeName = getNode()==NAryTree.TAU ? "tau" : String.valueOf(getNode());
        String operatorName = TreeUtils.getOperator(connector);
        switch (op){
            case CONVERT_TO:
                name = "convert to " + operatorName;
                if (connector == NAryTree.LOOP){
                    if (left){ // mandatory
                        name += " forward (+)";
                    } else {
                        name += " backward (*)";
                    }
                }
                break;
            case INSERT:
                name = "insert ("+nodeName+") in "+operatorName;
                if (connector == NAryTree.SEQ){
                    name += left?" before":" after";
                }
                break;
            case REMOVE:
                name = "remove";
                break;
            default:
                name = "unknown!";
        }

        return "Op{" +name+ "}";
    }

    public void setLeft(boolean left) {
        this.left = left;
    }
}
