package org.processmining.generalizedconformance.visualizer;

import org.processmining.generalizedconformance.algorithms.miner.QualityCriterion;

import javax.swing.*;
import java.awt.*;
import java.util.Map;

public class FitnessVisualizer extends JPanel {

    Map<QualityCriterion, Object> quality;

    private JProgressBar sim_l2, sim_lm, sim_m2;

    private JProgressBar fitness, precision, generalization;

    public FitnessVisualizer(){
        JLabel sim_l2_label = new JLabel("sim_l2:");
        sim_l2_label.setToolTipText("Similarity between original log and optimal log.");
        JLabel sim_lm_label = new JLabel("sim_lm:");
        sim_lm_label.setToolTipText("Similarity between optimal log and (current) optimal model.");
        JLabel sim_m2_label = new JLabel("sim_m2:");
        sim_m2_label.setToolTipText("Similarity between optimal model and original model.");

        JLabel fitness_label = new JLabel("fitness:");
        fitness_label.setToolTipText("Replay fitness between optimal log and optimal model.");
        JLabel precision_label = new JLabel("precision:");
        precision_label.setToolTipText("Replay precision between optimal log and optimal model.");
        JLabel generalization_label = new JLabel("generalization:");
        generalization_label.setToolTipText("Generalization of optimal model with respect to optimal log.");

        sim_l2 = new JProgressBar();
        sim_l2.setMinimumSize(new Dimension(100,20));
        sim_lm = new JProgressBar();
        sim_lm.setMinimumSize(new Dimension(100,20));
        sim_m2 = new JProgressBar();
        sim_m2.setMinimumSize(new Dimension(100,20));
        fitness = new JProgressBar();
        fitness.setMinimumSize(new Dimension(100,20));
        precision = new JProgressBar();
        precision.setMinimumSize(new Dimension(100,20));
        generalization = new JProgressBar();
        generalization.setMinimumSize(new Dimension(100,20));

        JPanel container = new JPanel();
        container.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        container.add(sim_l2_label, c);
        c.gridy = 1;
        container.add(sim_lm_label, c);
        c.gridy = 2;
        container.add(sim_m2_label, c);
        c.gridy = 3;
        c.gridwidth = 2;
        c.weighty = 1.0;   //request any extra vertical space
        //c.insets = new Insets(10,0,0,0);  //top padding
        container.add(new JLabel("log-model-alignment:"), c);
        c.gridwidth = 1;
        c.gridy = 4;
        container.add(fitness_label, c);
        c.gridy = 5;
        container.add(precision_label, c);
        c.gridy = 6;
        container.add(generalization_label, c);


        c.gridy = 0;
        c.gridx = 1;
        c.fill =  GridBagConstraints.HORIZONTAL;
        container.add(sim_l2,c);
        c.gridy = 1;
        container.add(sim_lm,c);
        c.gridy = 2;
        container.add(sim_m2,c);

        c.gridy = 4;
        container.add(fitness,c);
        c.gridy = 5;
        container.add(precision,c);
        c.gridy = 6;
        container.add(generalization,c);

        container.setPreferredSize(new Dimension(250,400));

        this.add(container);
    }

    public void update(Map<QualityCriterion, Object> quality){
        this.quality = quality;

        sim_l2.setValue((int)((Double)quality.get(QualityCriterion.LOG_SIMILARITY)*100));
        sim_l2.setToolTipText("<html>Similarity of input log and current log: "+quality.get(QualityCriterion.LOG_SIMILARITY) +
                "<br>(based on the string edit distance)</html>");
        sim_lm.setValue((int)((Double)quality.get(QualityCriterion.MODEL_LOG_SIMILARITY)*100));
        sim_lm.setToolTipText("<html>Similarity of current log and current model:" + quality.get(QualityCriterion.MODEL_LOG_SIMILARITY) +
                "<br>(arithmetic mean of fitness, precision, and generalization)</html>");
        sim_m2.setValue((int)((Double)quality.get(QualityCriterion.MODEL_SIMILARITY)*100));
        sim_m2.setToolTipText("<html>Similarity of current model and original model:" + quality.get(QualityCriterion.MODEL_SIMILARITY) +
                "<br>(based on the tree edit distance)</html>");

        fitness.setValue((int)((Double)quality.get(QualityCriterion.FITNESS)*100));
        fitness.setToolTipText("replay fitness: "+quality.get(QualityCriterion.FITNESS));
        precision.setValue((int)((Double)quality.get(QualityCriterion.PRECISION1)*100));
        precision.setToolTipText("replay precision: "+quality.get(QualityCriterion.PRECISION1));
        generalization.setValue((int)((Double)quality.get(QualityCriterion.GENERALIZATION1)*100));
        generalization.setToolTipText("generalization: "+quality.get(QualityCriterion.GENERALIZATION1));

        revalidate();
        repaint();
    }


}
