package org.processmining.generalizedconformance.visualizer;

import org.deckfour.xes.model.XLog;
import org.jgraph.JGraph;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.Pair;
import org.processmining.generalizedconformance.Utils;
import org.processmining.generalizedconformance.algorithms.miner.GeneralizedMiner;
import org.processmining.generalizedconformance.algorithms.miner.QualityCriterion;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.jgraph.visualization.ProMJGraphPanel;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.visualization.tree.TreeLayoutBuilder;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.PetrinetWithMarkings;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.util.Map;

public class GeneralizedMinerVisualizer implements ChangeListener {
	
	private JSlider modelTrust; 
	private JSlider logTrust;
	
	private double trustInModel = 1;
	private double trustInLog = 1;
	
	private JPanel graphPanel;
	
	private FitnessVisualizer fitnessVisualizer;

	private JPanel panel;
	private GeneralizedMiner miner;
	
	private PluginContext context;
	
	@Plugin(name = "Generalized Miner Visualizer", returnLabels = { "Visualized Generalized Miner" }, returnTypes = { JComponent.class }, parameterLabels = { GeneralizedMiner.PARAMETER_LABEL }, userAccessible = true)
	@Visualizer
	public static JComponent visualize(PluginContext context, GeneralizedMiner gMiner) {
		GeneralizedMinerVisualizer visualizer = new GeneralizedMinerVisualizer();
		return visualizer.getVisualization(context, gMiner);
	}

	private JComponent getVisualization(PluginContext context, GeneralizedMiner gMiner) {
		miner = gMiner;
		panel = new JPanel(new BorderLayout());
		this.context = context;
		
		modelTrust = new JSlider(0, 100, 100);
		logTrust = new JSlider(0, 100, 100);
		
		JComponent controls = new JPanel();
		controls.add(new JLabel("trust in model:"));
		controls.add(modelTrust);
		controls.add(new JLabel(" "));
		controls.add(new JLabel("trust in log:"));
		controls.add(logTrust);
		panel.add(controls, BorderLayout.SOUTH);
		
		modelTrust.addChangeListener(this);
		logTrust.addChangeListener(this);
		
		fitnessVisualizer = new FitnessVisualizer();
		panel.add(fitnessVisualizer, BorderLayout.EAST);

        updateModelAndLogPair();
		return panel;
	}

	private void updateGraph(ProcessTree tree, PetrinetWithMarkings petrinetWithMarkings) {
		TreeLayoutBuilder builder = new TreeLayoutBuilder(tree);
		JGraph graph = builder.getJGraph();
		
		if (graphPanel != null){
			panel.remove(graphPanel);
		}
		graphPanel = new JPanel(new GridLayout(2, 1));
		graphPanel.add(new JScrollPane(graph));
		
		ProMJGraphPanel newGraphPanel = null;
		if (context == null){
			newGraphPanel = ProMJGraphVisualizer.instance().visualizeGraphWithoutRememberingLayout(petrinetWithMarkings.petrinet);
		} else {
			newGraphPanel = ProMJGraphVisualizer.instance().visualizeGraph(context, petrinetWithMarkings.petrinet);
		}
		graphPanel.add(new JScrollPane(newGraphPanel));
		
		
		panel.add(graphPanel, BorderLayout.CENTER);
		panel.repaint();
		panel.revalidate();
	}

	public void stateChanged(ChangeEvent e) {
		boolean changed = false;
		if (e.getSource().equals(modelTrust)){
			double trust = modelTrust.getValue() / 100.;
			if (trustInModel != trust){
				changed = true;
			}
			trustInModel = trust;
		} else if (e.getSource().equals(logTrust)){
			double trust = logTrust.getValue() / 100.;
			if (trustInLog != trust){
				changed = true;
			}
			trustInLog = trust;
		}
		if (changed){
			updateModelAndLogPair();
		}
	}

	private void updateModelAndLogPair() {
		Pair<XLog,ProcessTree> logTreePair = miner.getFittingPair(trustInLog, trustInModel);
		System.out.println(logTreePair.getSecond());
		updateGraph(logTreePair.getSecond(), miner.getLastModelPetriNet());
		
		// align log and model:
		Map<QualityCriterion, Object> quality = Utils.getDistance(miner.getLastModelPetriNet(), logTreePair.getFirst());
		quality.put(QualityCriterion.LOG_SIMILARITY, Utils.getTraceSimilarity(miner.getOrigLog(), logTreePair.getFirst()));
		quality.put(QualityCriterion.MODEL_SIMILARITY, Utils.getModelSimilarity(miner.getOrigTree(), logTreePair.getSecond()));
		quality.put(QualityCriterion.MODEL_LOG_SIMILARITY, ((double) quality.get(QualityCriterion.FITNESS) + (double) quality.get(QualityCriterion.GENERALIZATION1) + (double) quality.get(QualityCriterion.PRECISION1))/ 3);

		fitnessVisualizer.update(quality);
		
		System.out.println("new fitness: "+quality.toString());
	}
}
