package org.processmining.generalizedconformance.visualizer;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.*;

import com.mxgraph.layout.hierarchical.mxHierarchicalLayout;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.view.mxGraph;
import org.deckfour.xes.classification.XEventClasses;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import org.processmining.generalizedconformance.algorithms.mover.TreeUtils;
import org.processmining.plugins.etm.model.narytree.NAryTree;

public class NAryTreeVisualization {

    public static Vector<DefaultGraphCell> vcells = new Vector<DefaultGraphCell>();
    public static HashMap<Integer, Integer> ReversedEdges = new HashMap<Integer, Integer>();
    public static DefaultGraphCell[] cells;

    public static void showTree(NAryTree tree, XEventClasses classes, String title){
        mxGraphComponent graph = convertToJGraph(tree, classes);

        graph.setPreferredSize(new Dimension(800,800));
        ScrollPane scrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        scrollPane.add(graph);
        scrollPane.setPreferredSize(new Dimension(900,900));

        JFrame frame = new JFrame("tree: '"+title+"'");
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.getContentPane().add(scrollPane);
        frame.pack();
        frame.revalidate();
        frame.repaint();
        frame.setVisible(true);
    }

    public static mxGraphComponent convertToJGraph(NAryTree t, XEventClasses classes) {
        com.mxgraph.view.mxGraph graph = new mxGraph();

        Object parent = graph.getDefaultParent();

        Object[] nodes = new Object[t.size()];

        graph.getModel().beginUpdate();
        try
        {
            for (int i = 0; i < t.size(); i++) {
                nodes[i] = graph.insertVertex(parent, null, getNodeLabel(t, i, classes), 20, 20, 40,
                        25);
            }
            depthFirstConnect(parent, t, classes, 0, graph, nodes);
        }
        finally
        {
            graph.getModel().endUpdate();
        }
        mxGraphComponent graphComponent = new mxGraphComponent(graph);

        graph.getModel().beginUpdate();

        mxHierarchicalLayout layout = new mxHierarchicalLayout(graph);
        layout.execute(parent);


        graph.getModel().endUpdate();

        return graphComponent;
    }

    public static String getNodeLabel(NAryTree tree, int node, XEventClasses classes){
        String label;
        if (tree.isLeaf(node)) {
            short type = tree.getType(node);
            if (type == NAryTree.TAU) {
                label = "TAU";
            } else if (classes != null) {
                if (classes.size() < type){
                    System.err.println("why is this happening in NAryTreeVisualization.getNodeLabel()??");
                    label = "unknown";
                } else {
                    label = classes.getByIndex(type).getId();
                }
            } else {
                label = "" + type;
            }
        } else {
            label = TreeUtils.getOperator(tree.getType(node));
        }
        return label;
    }


    private static void depthFirstConnect(Object parent, NAryTree tree, XEventClasses classes, int n, mxGraph graph, Object[] nodes) {
        Object graphNode = nodes[n];

        for (int c = 0; c < tree.nChildren(n); c++) {
            int childPos = tree.getChildAtIndex(n, c);
            depthFirstConnect(parent, tree, classes, childPos, graph, nodes);
            Object childNode = nodes[childPos];
            graph.insertEdge(parent, null, "", graphNode, childNode);
        }
    }

    /**
     * @param tree
     * @return the node <b>n<\b> converted to a cell
     */
    private static DefaultGraphCell ConvertNode(NAryTree tree, XEventClasses classes, int node) {
        if (tree.isLeaf(node)) {

            short type = tree.getType(node);
            String typeString;
            if (type == NAryTree.TAU) {
                typeString = "TAU";
            } else if (classes != null) {
                typeString = classes.getByIndex(type).toString();
            } else {
                typeString = "" + type;
            }

            DefaultGraphCell cell = new DefaultGraphCell("<html>" + typeString + "</html>");
            GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0, 0, 40, 40));
            GraphConstants.setAutoSize(cell.getAttributes(), true);
            GraphConstants.setBorderColor(cell.getAttributes(), Color.black);
            return cell;
        } else {
            //Set the operator label
            String label = "";
            switch (tree.getType(node)) {
                case NAryTree.SEQ :
                    label = "->";
                    //					label = "<html>&rarr;</html>";
                    break;
                case NAryTree.REVSEQ :
                    label = "<-";
                    //					label = "<html>&rarr;</html>";
                    break;
                case NAryTree.XOR :
                    label = "X";
                    break;
                case NAryTree.AND :
                    label = "/\\";
                    break;
                case NAryTree.OR :
                    label = "\\/";
                    break;
                case NAryTree.ILV :
                    label = "<->";
                    break;
                case NAryTree.LOOP :
                    label = "L";
                    break;
                default :
                    label = "$%@$";
                    break;
            }

            int width = 20;
            int height = 20;
            //int pythagoras = (int) Math.sqrt(Math.pow(width / 2, 2) + Math.pow(height / 2, 2));
            DefaultGraphCell cell = new DefaultGraphCell(label);
            GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0, 0, width, height));
            //GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0, 0, pythagoras, pythagoras));
            //GraphConstants.setSize(cell.getAttributes(), new Dimension(width, height));
            GraphConstants.setAutoSize(cell.getAttributes(), false);
            GraphConstants.setBorderColor(cell.getAttributes(), Color.black);
            return cell;
        }
    }
}
